/*
	Pinguino + MOD-RFID125-BOX
	example.
	
	Instructions will appear on the terminal.
	Also, refer to the project README for
	more information on the boards and
	how to customize the example.
	
	If you have any questions, email
	support@olimex.com
	
	Olimex, January 2013
	http://www.olimex.com/dev
	
*/

#include <serial.c>
#include <__cdc.c>

#if ( defined(PIC32_PINGUINO_OTG) || defined(PIC32_PINGUINO) || defined(PIC32_PINGUINO_MICRO) )

	//Definitions for 
	// -- PIC32-PINGUINO
	// -- PIC32-PINGUINO-OTG
	// -- PIC32-PINGUINO-MICRO
	
	#define rf_port UART2      //The UEXT is on UART2
	#define rf_baud 9600
	#define BUTTON1INIT TRISDbits.TRISD0 = 1
	
	//YELLOW LED is D1
	#define YLEDINIT   TRISDCLR = 0x02;
	#define YLED1      PORTDSET = 0x02;
	#define YLED0      PORTDCLR = 0x02;
	#define YLEDSWITCH PORTD   ^= 0x02;
	
	//GREEN LED is G6
	#define GLEDINIT  	TRISGCLR = 0x40;
	#define GLED1 		PORTGSET = 0x40;
	#define GLED0 		PORTGCLR = 0x40;
	#define GLEDSWITCH  PORTG   ^= 0x40;
	
	#define YELLOW 0
	#define GREEN  1
	#define BOTH   2
#endif

/*  Prototypes and useful functions */


	//Blinks an LED five times - mode is YELLOW, GREEN or BOTH
	void blinkled(unsigned char mode);

	//Catches a string and writes to the buffer
	//In this example the function skips any <CR>, <LF>, '-' or '>'
	//at the beginning of the string in order to isolate the tag ID
	void RFID_get_response(unsigned char uport);
	
	//checks if the given tag address is present in the list of 'allowed' tags
	//returns the tag number if it is and -1 if not
	volatile char check_tag_data(volatile unsigned char * tag_id);
	
	//Adds a tag to the database
	boolean add_new_tag();	
		  
	//A macro that waits for a key via CDC
	//Works with all pinguinos that support CDC
	//Syntax:
	//   WAIT;   //-> pauses the program and waits
				 //   until a key is pressed 
		
	//WAIT Macro 
	#define WAIT wait_123()
	char ch123;
	
	void wait_123() __attribute ((noinline));
	
	void wait_123(){ ch123 = CDCgetkey();}
	//END Wait Macro
	
	// Sends a single line of text to the terminal via CDC
	void message(char * m){ CDCprintln(m);}

/*	Global variables  */

	#define SERIAL_BUFFER_SIZE 80
	volatile unsigned char serial_buffer[SERIAL_BUFFER_SIZE];

	#define MAX_TAGS 5
	#define MAX_TAG_LENGTH 15
	volatile unsigned char tag_id_data[MAX_TAGS][MAX_TAG_LENGTH];
	volatile unsigned char tag_counter=0;
	volatile unsigned char tag_id_buf[MAX_TAG_LENGTH];
	
/*	    Interrupt routine definition  */

	// Connect the wrapper to vector 4 (Timer1 overflow)
	void ISR_wrapper_vector_4(void) __attribute__ ((section (".vector_4")));

	// The wrapper calls the "Tmr1Interrupt" function
	// each time an interrupt occurs
	void ISR_wrapper_vector_4(void) { Tmr1Interrupt(); }

	//We declare the function as an interrupt routine
	void Tmr1Interrupt(void) __attribute__ ((interrupt));
	
	// configure timer 1
	void init_timer1(void)
	{
		IntConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);	// interrupt mode (interrupt.c)
		T1CON=0;			// reset timer 1 configuration
		TMR1=0;				// reset timer 1 counter register
		PR1=0x9896;			// define the period (an interrupt will occur ~ 128 times a second)
		IPC1SET=0x7;		// select interrupt priority and sub-priority
		IFS0CLR=0x10;		// clear interrupt flag
		IEC0SET=0x10;		// enable timer 1 interrupt
		T1CONCLR=0x30;	  	// set prescaler to 1:1
		T1CONSET=0x10;		// set prescaler to 1:8
		T1CONSET=0x8000; 	// turn TIMER1 ON
	}
	
	
// The interrupt routine code			
void Tmr1Interrupt(void)
{
	T1CONCLR = (1<<15);  //Stop Timer 1
	
	//if anything is available on the serial port...
	if(SerialAvailable(rf_port))
	{	
		//... catch the string and isolate the tag ID
		RFID_get_response(rf_port);
		
		//put the string in the buffer for the last recorded tag ID
		strcpy(tag_id_buf, serial_buffer);
				
		//check if the tag is in the list of 'allowed' tags
		//resp is a number between 1 and MAX_TAGS if it is an
		//allowed tag and -1 if not
		volatile char resp = 0;
		resp = check_tag_data(tag_id_buf);
				
		if(resp==-1)
		{
			// do something if the tag is not recognized
			message("Unrecognized tag");
		}else
			{
				if(resp==0) message("ERROR!"); // there was an error if resp remained 0
				else
				{
					//do something if the tag is found in the list
					CDC.print("Tag number "); CDC.printf("%d", resp); CDC.print(" recognized.");
					message("");
				}
			}
	}
	
	// Start Timer 1 again
	init_timer1();
	
}

void setup()
{	
	WAIT;
	
	/* Init the LEDs and Button on the Pinguino board*/
		YLEDINIT;
		GLEDINIT;	
		YLED0;
		GLED0;
		BUTTON1INIT;
	
	// Open a serial communication from the board to the module
		SerialConfigure(rf_port, UART_ENABLE, UART_RX_TX_ENABLED, rf_baud);
	
	
	message("Connection to MOD-RFID125-BOX is open.");
	message("1 - Init the list of allowed tags: ");
	
	//Setting up the tag database
	char add_new = 1;
	
	do
	{
		if( add_new_tag() )
		{
			CDC.print("Tag "); CDC.print(tag_id_data[tag_counter-1]); CDC.print(" added.");
			message("");
			
			if(tag_counter == MAX_TAGS)
			{
				message("Maximum number of tags reached. No more tags can be added.");
				add_new = 0;
			}
			else
			{
				message("Do you want to add a new tag? Y/N");
				char m; m = CDC.getKey(); message("");
				if( m == 'Y' || m == 'y' )
					add_new = 1;
				else
					add_new = 0;
			}		
				
		}else
		 {
		   message("There was an error while adding the tag...");
		 }
		
		
	}while(add_new == 1);

	message("Tag data initialization is complete.");
	
	message("The RFID device will now continuously scan and for tags");
	message("and alert you on the terminal. Press a key to start...");
	
	WAIT;
	
	message("2 - 'Recognize tag' mode started");
	
	init_timer1();
		
}

void loop()
{}

boolean add_new_tag()
{
	if(tag_counter<MAX_TAGS)
	{
		message("Get the tag within reading distance. (<10cm)");
		
		//catch the string containing the tag number
		RFID_get_response(rf_port);
		
		strcpy(tag_id_buf, serial_buffer);
		
		//Add the address to the array containing all current tags
		strcpy(tag_id_data[tag_counter], tag_id_buf);
		tag_counter++;
					
		return 1;	
		
	}else
	  {		
		return 0;
	  }
}
	
void RFID_get_response(unsigned char uport)
{
	volatile unsigned char c, c_skip, i=0;

	//Skip any <CR>, <LF>, '-', or '>' before the tag ID
	do{
		c_skip = SerialGetKey(uport);
	  }while(c_skip == 0x0d || c_skip == 0x0a || c_skip == '>' || c_skip == '-');
	
	serial_buffer[i] = c_skip;
	i++;
	
	// The rest of the code is based on the 
	// SerialGetString function in the 
	// <serial.c> library
		do {
				c = SerialGetKey(uport);
				
			    // Stop when you meet the <CR> character
				if( c != '\r')
				{	
					serial_buffer[i] = c;
					i++;
				}
						
			}while( c != '\r' );
		serial_buffer[i] = '\0';
		
	 //Catch the <LF> character
		c = SerialGetKey(uport);

}

volatile char check_tag_data(volatile unsigned char * tag_id)
{
	volatile unsigned char i;
	for(i=0; i<tag_counter; ++i)
	{
		if( strcmp(tag_id, tag_id_data[i]) == 0 )
			return (i+1);	//if found, return the tag number
	}
	//if not found, return -1
	return -1;
}

void blinkled(unsigned char mode)
{		
	//Turn off all LEDs
	YLED0;
	GLED0;
	
	char i;
	
	// Blink
	switch(mode)
	{
		case YELLOW:
			for(i=0; i<10;i++)
			{
				YLEDSWITCH;
				delay(100);
			}
			break;
		
		case GREEN:
			for(i=0; i<10;i++)
			{
				GLEDSWITCH;
				delay(100);
			}
			break;
		
		case BOTH:
			for(i=0; i<10;i++)
			{
				YLEDSWITCH;
				GLEDSWITCH;
				delay(100);
			}
			break;
	}	
					
				
}